#include <SoftwareSerial.h>
#include <TM1650.h>
#define RXMAX 128
#define RXID 100 //Message ID from the transmitter
#define TIMEOUT 5000
#define BUTTONTIME 200
#define SW1 A1
#define POT A0
#define SW2 A2
#define SW3 2
#define SW4 3

TM1650 module(A4, A5, 4);  // SDA=A4, SCL=A5
SoftwareSerial mySerial(5, 4); // RX, TX

bool toggle = true; //LED blink

//Varibales for reading Reyax Lora message
int i = 0; //last rx position
int commaCount = 0;
int lastCommaPos = 0; 
char rx[RXMAX] = {0}; //rx buffer

//Parsed Data From Reyax Lora
int gpsSpeed = 0;
int rssi = 0;
int snr = 0;

void displayNumber(int temp, bool hundredths = false)
{  
  module.setDisplayDigit(temp / 100, 0, hundredths ? true : false);
  temp = temp - ((temp / 100) * 100);
  module.setDisplayDigit((temp / 10), 1, hundredths ? false : true);
  temp = temp - ((temp / 10) * 10);
  module.setDisplayDigit((temp), 2, false);
}

void displayNumberNoDecimal(int temp)
{  
  module.setDisplayDigit(temp / 100, 0, false);
  temp = temp - ((temp / 100) * 100);
  module.setDisplayDigit((temp / 10), 1, false);
  temp = temp - ((temp / 10) * 10);
  module.setDisplayDigit((temp), 2, false);
}

void ParseData()
{
  if(commaCount == 1)
  { 
      int address = atoi(rx);
      if(address != RXID)
      {
        Serial.println("Rx from illegal sender");
        commaCount = -1; //wait to reset
      }
  }
  else if(commaCount == 3)
  {
      //Serial.print("Speed: ");
      gpsSpeed = atoi(&rx[lastCommaPos]);
      //Serial.println(gpsSpeed);
  }
  else if(commaCount == 4)
  {
      //Serial.print("rssi: ");
      rssi = atoi(&rx[lastCommaPos]);
      //Serial.println(rssi);
  }
  else if(commaCount == 5)
  {
      //Serial.print("snr: ");
      snr = atoi(&rx[lastCommaPos]);
      //Serial.println(snr);
  }
  else
  {
    Serial.println("unknown");
  }
}

void setup() {
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, LOW);
  pinMode(SW1, INPUT_PULLUP);
  pinMode(SW2, INPUT_PULLUP);
  pinMode(SW3, INPUT_PULLUP);
  pinMode(SW4, INPUT_PULLUP);
  pinMode(POT, INPUT_PULLUP);

  //Display module
  module.clearDisplay();
  delay(1000);  

  //Roll up to 1.1 like in the movie
  for(int nVal=0; nVal<=11; nVal++)     
  {
    displayNumber(nVal);
    delay(100);
  }
  
  displayNumber(0);
  
  while (!Serial); // wait for Serial to be ready
  Serial.begin(57600); // The serial port for the Arduino IDE port output
  delay(2000);
  Serial.println("Serial Ready");
  
  // set the data rate for the SoftwareSerial port
  mySerial.begin(57600);
  delay(2000);  
  Serial.println("Software Serial Reyax Ready");
}


void loop() { 
  static bool connectionActive = false;
  static bool displayUpdated = false;
  static long timeout = millis();
  static long buttonTime = millis();

   
  if (mySerial.available()) 
  {
    connectionActive = true;
    timeout = millis();
    toggle = !toggle;  
    digitalWrite(LED_BUILTIN, toggle?HIGH:LOW);
    char c = mySerial.read();
    //Serial.write(c);
    
    if(c == '=')
    {
      commaCount = 0;
      i = 0;
      lastCommaPos = 0;
      memset(rx, 0, RXMAX );
    }
    else if((c == ',' || c == '\r') && commaCount != -1)
    {
      commaCount++;
      ParseData();
      lastCommaPos = i;
    }
    else
    {
      rx[i++] = c;
    }
  }
  else
  {
    if(millis() - timeout > TIMEOUT)
    {
      connectionActive = false;
      timeout = millis();
    }
  }
 
  if(millis() - buttonTime > BUTTONTIME)
  {
    buttonTime = millis();
    
    if( digitalRead(SW1) == LOW)
    {
      module.setDisplayToString("   ");
      module.clearDisplayDigit(1, true);
      displayUpdated = true;
      Serial.println("SW1");
    }
    
    else if(digitalRead(SW2) == LOW)
    {
      displayNumberNoDecimal(abs(rssi));
      displayUpdated = true;
      Serial.println("SW2");
      Serial.print("rssi:");
      Serial.println(rssi);

    }
    
    else if(digitalRead(SW3) == LOW)
    {
      displayNumberNoDecimal(snr);
      displayUpdated = true;
      Serial.println("SW3");
      Serial.print("SNR:");
      Serial.println(snr);
    }
    
    else if(digitalRead(SW4) == LOW)
    {
      displayUpdated = true;
      Serial.println("SW4");
    }

    else if(connectionActive == false)
    {
      module.setDisplayToString("---");
      Serial.println("Lost Lora Comms");
      displayUpdated = true;
    }
    
    else
    {
      displayUpdated = false;
    }

    int pot = analogRead(POT);
    if(pot < 90) module.setupDisplay(true, 0);
    else if(pot < 180) module.setupDisplay(true, 3);
    else module.setupDisplay(true, 6);
    Serial.println(analogRead(POT));
  }

  if(displayUpdated == false)
  {
    if(gpsSpeed < 50) // 0.50 mph
      displayNumber(gpsSpeed, true);
    else
      displayNumber(gpsSpeed / 10.0);
  }
  
  if (Serial.available()) 
  {
    mySerial.write(Serial.read());
  }

  
}
